# Copyright (c) 2023 Darren Erik Vengroff
"""Places in Utah."""


ALPINE_CITY = "00540"
ALTA_TOWN = "00650"
ALTAMONT_TOWN = "00760"
ALTON_TOWN = "00870"
AMALGA_TOWN = "01090"
AMERICAN_FORK_CITY = "01310"
ANETH_CDP = "01530"
ANNABELLA_TOWN = "01750"
ANTIMONY_TOWN = "01860"
APPLE_VALLEY_TOWN = "01905"
AURORA_CITY = "02740"
AVON_CDP = "02960"
BALLARD_TOWN = "03345"
BEAR_RIVER_CITY_CITY = "03950"
BEAVER_CITY = "04060"
BENJAMIN_CDP = "04720"
BENSON_CDP = "05050"
BERYL_JUNCTION_CDP = "05380"
BICKNELL_TOWN = "05490"
BIG_WATER_TOWN = "05534"
BLANDING_CITY = "06370"
BLUEBELL_CDP = "06590"
BLUFF_TOWN = "06700"
BLUFFDALE_CITY = "06810"
BONANZA_CDP = "06920"
BOULDER_TOWN = "07470"
BOUNTIFUL_CITY = "07690"
BRIAN_HEAD_TOWN = "08020"
BRIGHAM_CITY_CITY = "08460"
BRIGHTON_TOWN = "08570"
BRYCE_CANYON_CITY_TOWN = "08787"
CACHE_CDP = "09725"
CANNONVILLE_TOWN = "10330"
CARBONVILLE_CDP = "10440"
CASTLE_DALE_CITY = "10660"
CASTLE_VALLEY_TOWN = "11000"
CEDAR_CITY_CITY = "11320"
CEDAR_FORT_TOWN = "11430"
CEDAR_HIGHLANDS_CDP = "11435"
CEDAR_HILLS_CITY = "11440"
CENTERFIELD_TOWN = "11870"
CENTERVILLE_CITY = "11980"
CENTRAL_CDP = "12200"
CENTRAL_VALLEY_TOWN = "12260"
CHARLESTON_TOWN = "12420"
CIRCLEVILLE_TOWN = "12970"
CLARKSTON_TOWN = "13190"
CLAWSON_TOWN = "13300"
CLEAR_CREEK_CDP = "13740"
CLEARFIELD_CITY = "13850"
CLEVELAND_TOWN = "14070"
CLINTON_CITY = "14290"
COALVILLE_CITY = "14840"
COPPERTON_METRO_TOWNSHIP = "15720"
CORINNE_CITY = "15830"
CORNISH_TOWN = "15940"
COTTONWOOD_HEIGHTS_CITY = "16270"
COVE_CDP = "16490"
DAMMERON_VALLEY_CDP = "18090"
DANIEL_TOWN = "18140"
DELTA_CITY = "18910"
DESERET_CDP = "19020"
DEWEYVILLE_TOWN = "19680"
DRAPER_CITY = "20120"
DUCHESNE_CITY = "20340"
DUGWAY_CDP = "20450"
DUTCH_JOHN_TOWN = "20780"
EAGLE_MOUNTAIN_CITY = "20810"
EAST_BASIN_CDP = "20850"
EAST_CARBON_CITY = "20890"
ECHO_CDP = "21880"
EDEN_CDP = "21990"
ELBERTA_CDP = "22210"
ELK_RIDGE_CITY = "22370"
ELMO_TOWN = "22540"
ELSINORE_TOWN = "22650"
ELWOOD_TOWN = "22760"
EMERY_TOWN = "22870"
EMIGRATION_CANYON_METRO_TOWNSHIP = "22875"
ENOCH_CITY = "23200"
ENTERPRISE_CDP = "23310"
ENTERPRISE_CITY = "23420"
EPHRAIM_CITY = "23530"
ERDA_CDP = "23640"
ESCALANTE_CITY = "23750"
EUREKA_CITY = "24080"
FAIRFIELD_TOWN = "24410"
FAIRVIEW_CITY = "24630"
FARMINGTON_CITY = "24740"
FARR_WEST_CITY = "24850"
FAYETTE_TOWN = "25070"
FERRON_CITY = "25180"
FIELDING_TOWN = "25290"
FILLMORE_CITY = "25510"
FLAMING_GORGE_CDP = "25910"
FORT_DUCHESNE_CDP = "26610"
FOUNTAIN_GREEN_CITY = "26720"
FRANCIS_TOWN = "26940"
FREMONT_CDP = "27270"
FRUIT_HEIGHTS_CITY = "27490"
GARDEN_CDP = "27902"
GARDEN_CITY_TOWN = "27930"
GARLAND_CITY = "28150"
GENOLA_TOWN = "28810"
GLENDALE_TOWN = "29360"
GLENWOOD_TOWN = "29580"
GOSHEN_TOWN = "30130"
GRANITE_CDP = "30900"
GRANTSVILLE_CITY = "31120"
GREEN_RIVER_CITY = "31670"
GUNNISON_CITY = "32660"
HALCHITA_CDP = "33002"
HALLS_CROSSING_CDP = "33027"
HANKSVILLE_TOWN = "33100"
HARRISVILLE_CITY = "33540"
HATCH_TOWN = "33760"
HEBER_CITY = "34200"
HELPER_CITY = "34530"
HENEFER_TOWN = "34640"
HENRIEVILLE_TOWN = "34750"
HERRIMAN_CITY = "34970"
HIDEOUT_TOWN = "35120"
HIGHLAND_CITY = "35190"
HILDALE_CITY = "35300"
HINCKLEY_TOWN = "35740"
HOBBLE_CREEK_CDP = "35823"
HOLDEN_TOWN = "35960"
HOLLADAY_CITY = "36070"
HONEYVILLE_CITY = "36290"
HOOPER_CITY = "36400"
HOWELL_TOWN = "36620"
HOYTSVILLE_CDP = "36730"
HUNTINGTON_CITY = "36950"
HUNTSVILLE_TOWN = "37060"
HURRICANE_CITY = "37170"
HYDE_PARK_CITY = "37390"
HYRUM_CITY = "37500"
INDEPENDENCE_TOWN = "37690"
INTERLAKEN_TOWN = "37924"
IVINS_CITY = "38710"
JENSEN_CDP = "39040"
JOSEPH_TOWN = "39370"
JUNCTION_TOWN = "39590"
KAMAS_CITY = "39810"
KANAB_CITY = "39920"
KANARRAVILLE_TOWN = "40030"
KANOSH_TOWN = "40250"
KAYSVILLE_CITY = "40360"
KEARNS_METRO_TOWNSHIP = "40470"
KENILWORTH_CDP = "41020"
KINGSTON_TOWN = "41350"
KOOSHAREM_TOWN = "41680"
LA_SAL_CDP = "43110"
LA_VERKIN_CITY = "43440"
LAKE_SHORE_CDP = "42230"
LAKETOWN_TOWN = "42560"
LAPOINT_CDP = "42890"
LAYTON_CITY = "43660"
LEAMINGTON_TOWN = "43880"
LEEDS_TOWN = "44100"
LEHI_CITY = "44320"
LEVAN_TOWN = "44650"
LEWISTON_CITY = "44760"
LIBERTY_CDP = "44870"
LINDON_CITY = "45090"
LOA_TOWN = "45530"
LOGAN_CITY = "45860"
LYMAN_TOWN = "46410"
LYNNDYL_TOWN = "46740"
MAESER_CDP = "47180"
MAGNA_METRO_TOWNSHIP = "47290"
MANILA_TOWN = "47620"
MANTI_CITY = "47730"
MANTUA_TOWN = "47840"
MAPLETON_CITY = "47950"
MARION_CDP = "48170"
MARRIOTT_SLATERVILLE_CITY = "48300"
MARYSVALE_TOWN = "48610"
MAYFIELD_TOWN = "48720"
MEADOW_TOWN = "48830"
MENDON_CITY = "49160"
MEXICAN_HAT_CDP = "49380"
MIDVALE_CITY = "49710"
MIDWAY_CITY = "49820"
MILFORD_CITY = "50040"
MILLCREEK_CITY = "50150"
MILLVILLE_CITY = "50370"
MINERSVILLE_TOWN = "50590"
MOAB_CITY = "50700"
MODENA_CDP = "50810"
MONA_CITY = "51140"
MONROE_CITY = "51360"
MONTEZUMA_CREEK_CDP = "51470"
MONTICELLO_CITY = "51580"
MORGAN_CITY = "51910"
MORONI_CITY = "52130"
MOUNT_PLEASANT_CITY = "53010"
MOUNTAIN_GREEN_CDP = "52460"
MURRAY_CITY = "53230"
MYTON_CITY = "53340"
NAPLES_CITY = "53560"
NAVAJO_MOUNTAIN_CDP = "53890"
NEOLA_CDP = "54110"
NEPHI_CITY = "54220"
NEW_HARMONY_TOWN = "54440"
NEWCASTLE_CDP = "54330"
NEWTON_TOWN = "54550"
NIBLEY_CITY = "54660"
NORTH_LOGAN_CITY = "54990"
NORTH_OGDEN_CITY = "55100"
NORTH_SALT_LAKE_CITY = "55210"
OAK_CITY_TOWN = "55430"
OAKLEY_CITY = "55650"
OASIS_CDP = "55760"
OGDEN_CITY = "55980"
OLJATO_MONUMENT_VALLEY_CDP = "56440"
OPHIR_CDP = "56750"
ORANGEVILLE_CITY = "56860"
ORDERVILLE_TOWN = "57080"
OREM_CITY = "57300"
PALMYRA_CDP = "57630"
PANGUITCH_CITY = "57740"
PARADISE_TOWN = "57850"
PARAGONAH_TOWN = "57960"
PARK_CITY_CITY = "58070"
PAROWAN_CITY = "58510"
PAYSON_CITY = "58730"
PEOA_CDP = "59280"
PERRY_CITY = "59390"
PETER_CDP = "59555"
PINE_VALLEY_CDP = "60270"
PLAIN_CITY_CITY = "60710"
PLEASANT_GROVE_CITY = "60930"
PLEASANT_VIEW_CITY = "61150"
PLYMOUTH_TOWN = "61260"
PORTAGE_TOWN = "61590"
PRICE_CITY = "62030"
PROVIDENCE_CITY = "62360"
PROVO_CITY = "62470"
RANDLETT_CDP = "62910"
RANDOLPH_TOWN = "63020"
REDMOND_TOWN = "63240"
RICHFIELD_CITY = "63570"
RICHMOND_CITY = "63680"
RIVER_HEIGHTS_CITY = "64120"
RIVERDALE_CITY = "64010"
RIVERSIDE_CDP = "64230"
RIVERTON_CITY = "64340"
ROCKVILLE_TOWN = "64560"
ROCKY_RIDGE_TOWN = "64590"
ROOSEVELT_CITY = "64670"
ROY_CITY = "65110"
RUSH_VALLEY_TOWN = "65240"
SALEM_CITY = "65770"
SALINA_CITY = "65880"
SALT_LAKE_CITY_CITY = "67000"
SAMAK_CDP = "67400"
SANDY_CITY = "67440"
SANTA_CLARA_CITY = "67660"
SANTAQUIN_CITY = "67770"
SARATOGA_SPRINGS_CITY = "67825"
SCIPIO_TOWN = "67880"
SCOFIELD_TOWN = "67990"
SIGURD_TOWN = "68650"
SILVER_SUMMIT_CDP = "69280"
SMITHFIELD_CITY = "69640"
SNOWVILLE_TOWN = "69970"
SNYDERVILLE_CDP = "70190"
SOUTH_JORDAN_CITY = "70850"
SOUTH_OGDEN_CITY = "70960"
SOUTH_SALT_LAKE_CITY = "71070"
SOUTH_WEBER_CITY = "71180"
SOUTH_WILLARD_CDP = "71255"
SPANISH_FORK_CITY = "71290"
SPANISH_VALLEY_CDP = "71455"
SPRING_CITY_CITY = "71730"
SPRING_GLEN_CDP = "72060"
SPRING_LAKE_CDP = "72170"
SPRINGDALE_TOWN = "71840"
SPRINGVILLE_CITY = "72280"
ST_GEORGE_CITY = "65330"
STANSBURY_PARK_CDP = "72720"
STERLING_TOWN = "72940"
STOCKTON_TOWN = "73050"
SUMMIT_CDP = "74040"
SUMMIT_PARK_CDP = "74095"
SUNDANCE_CDP = "74260"
SUNSET_CITY = "74480"
SUTHERLAND_CDP = "74590"
SYRACUSE_CITY = "74810"
TABIONA_TOWN = "74920"
TAYLORSVILLE_CITY = "75360"
TEASDALE_CDP = "75470"
THATCHER_CDP = "75800"
THOMPSON_SPRINGS_CDP = "76180"
TIMBER_LAKES_CDP = "76280"
TOOELE_CITY = "76680"
TOQUERVILLE_CITY = "76900"
TORREY_TOWN = "77010"
TREMONTON_CITY = "77120"
TRENTON_TOWN = "77230"
TROPIC_TOWN = "77560"
TSELAKAI_DEZZA_CDP = "77697"
UINTAH_TOWN = "77890"
VERNAL_CITY = "80090"
VERNON_TOWN = "80200"
VEYO_CDP = "80310"
VINEYARD_TOWN = "80420"
VIRGIN_TOWN = "80530"
WALES_TOWN = "80860"
WALLSBURG_TOWN = "80970"
WANSHIP_CDP = "81190"
WASHINGTON_TERRACE_CITY = "82070"
WASHINGTON_CITY = "81960"
WELLINGTON_CITY = "82510"
WELLSVILLE_CITY = "82620"
WENDOVER_CITY = "82730"
WEST_BOUNTIFUL_CITY = "82840"
WEST_HAVEN_CITY = "82930"
WEST_JORDAN_CITY = "82950"
WEST_MOUNTAIN_CDP = "83252"
WEST_POINT_CITY = "83390"
WEST_VALLEY_CITY_CITY = "83470"
WEST_WOOD_CDP = "83730"
WHITE_CITY_METRO_TOWNSHIP = "84050"
WHITE_MESA_CDP = "84095"
WHITEROCKS_CDP = "84160"
WILLARD_CITY = "84710"
WOLF_CREEK_CDP = "84950"
WOODLAND_CDP = "85040"
WOODLAND_HILLS_CITY = "85050"
WOODRUFF_TOWN = "85260"
WOODS_CROSS_CITY = "85370"



# Copyright (c) 2023 Darren Erik Vengroff
"""Main module for the command line interface to censusdis."""
from typing import Optional
import argparse
import logging
import sys
from logging import getLogger

import geopandas as gpd
import matplotlib.pyplot as plt

from logargparser import LoggingArgumentParser

from censusdis.cli.yamlspec import DataSpec, PlotSpec

logger = getLogger(__name__)


def main():
    """Entry point for the CLI interface to censusdis."""
    parser = LoggingArgumentParser(logger, prog="censusdis")

    subparsers = parser.add_subparsers(
        parser_class=argparse.ArgumentParser,
        required=True,
        title="command",
        description="Choose one of the following commands.",
    )
    download_parser = subparsers.add_parser(
        "download", help="Download data from the U.S. Census API."
    )

    download_parser.add_argument(
        "--api-key",
        type=str,
        help="Optional API key. Alternatively, store your key in "
        "~/.censusdis/api_key.txt. It you don't have a key, you "
        "may get throttled or blocked. Get one from "
        "https://api.census.gov/data/key_signup.html",
    )
    download_parser.add_argument(
        "-o",
        "--output",
        type=str,
        required=True,
        help="Output file to store the data in. Format will be determined from the "
        "file extansion. .csv or .geojson (the latter if your spec has with_geometry: true.",
    )
    download_parser.add_argument("dataspec", type=str, help="A dataspec YAML file.")

    download_parser.set_defaults(func=download)

    plot_parser = subparsers.add_parser("plot", help="Plot data on a map.")

    data_group = plot_parser.add_mutually_exclusive_group(required=True)
    data_group.add_argument(
        "--dataspec",
        type=str,
        help="A data specification YAML file. "
        "If provided, data is downloaded from the U.S. Census API as in the download command.",
    )
    data_group.add_argument(
        "-i",
        "--input-data-file",
        type=str,
        help="Local file to load data from. This is normally a .geojson file "
        "that was previously downloaded with the download command.",
    )

    plot_parser.add_argument(
        "--api-key",
        type=str,
        help="Optional API key. Ignored if data is loaded from a local file "
        "with -i/--input-data-file. Alternatively, store your key in "
        "~/.censusdis/api_key.txt. It you don't have a key, you "
        "may get throttled or blocked. Get one from "
        "https://api.census.gov/data/key_signup.html",
    )

    plot_parser.add_argument(
        "plotspec", type=str, help="A plot specification YAML file."
    )
    plot_parser.add_argument(
        "-o",
        "--output",
        type=str,
        required=True,
        help="Output file to store the plotted map in. Format will be determined from the "
        "file extansion. .png or .jpeg typically.",
    )

    plot_parser.set_defaults(func=plot)

    args = parser.parse_args()

    args.func(args)


def _read_dataspec(dataspec_file: str) -> DataSpec:
    logger.info(f"Loading data spec from {dataspec_file}.")
    dataspec = DataSpec.load_yaml(dataspec_file)
    logger.info("Loaded.")

    if not isinstance(dataspec, DataSpec):
        logging.critical(
            f"{dataspec_file} does not contain YAML for a data spec. It should start with the tag '!DataSpec'"
        )
        sys.exit(1)

    return dataspec


def _download_data(dataspec: DataSpec, needs_geometry: bool, api_key: Optional[str]):
    if needs_geometry and not dataspec.with_geometry:
        logger.critical(
            "Data specification does not have `with_geometry: true`, but geometry is needed to save "
            "in .geojson format or to plot."
        )
        sys.exit(2)

    logger.info("Downloading data from the U.S. Census API.")
    df_or_gdf = dataspec.download(api_key=api_key)
    logger.info("Download complete.")

    return df_or_gdf


def download(args):
    """Execute the download command from the CLI."""
    logger.debug("Download command selected.")

    dataspec_file = args.dataspec
    output = args.output
    needs_geometry = output.endswith(".geojson")

    dataspec = _read_dataspec(dataspec_file)

    df_or_gdf = _download_data(dataspec, needs_geometry, args.api_key)

    logger.info(f"Writing data to {output}.")

    if output.endswith(".csv"):
        if dataspec.with_geometry:
            logger.warning(
                "Data with geometry being written to a csv file. You might prefer .geojson."
            )
        df_or_gdf.to_csv(output, index=False)
    elif output.endswith(".geojson"):
        df_or_gdf.to_file(output, driver="GeoJson")
    else:
        logger.warning(
            f"Unrecognized file type {output}. This might or might not work."
        )
        df_or_gdf.to_file(output)
    logger.info("Writing complete.")


def plot(args):
    """Execute the plot command from the CLI."""
    logger.debug("Plot command selected.")

    logger.info(f"Loading plot spec from {args.plotspec}.")

    plotspecs = PlotSpec.load_yaml(args.plotspec)

    logger.info("Loaded.")

    if args.input_data_file is not None:
        # Load from a local file.
        logger.info(f"Loading data from local file {args.input_data_file}.")
        gdf = gpd.read_file(args.input_data_file)
    else:
        # Load from a dataspec.
        dataspec_file = args.dataspec
        dataspec = _read_dataspec(dataspec_file)
        gdf = _download_data(dataspec, True, args.api_key)

    if isinstance(plotspecs, PlotSpec):
        plotspecs = [plotspecs]

    ax = None

    for plotspec in plotspecs:
        ax = plotspec.plot(gdf, ax=ax)

    fig = ax.get_figure()

    output_file = args.output

    logger.info(f"Saving plot to {output_file}.")
    fig.savefig(output_file)
    plt.close(fig)


if __name__ == "__main__":
    main()



# Copyright (c) 2023 Darren Erik Vengroff
"""Places in Washington."""


ABERDEEN_GARDENS_CDP = "00135"
ABERDEEN_CITY = "00100"
ACME_CDP = "00275"
ADDY_CDP = "00380"
AHTANUM_CDP = "00800"
AIRWAY_HEIGHTS_CITY = "00905"
ALBION_TOWN = "01010"
ALDER_CDP = "01045"
ALDERTON_CDP = "01150"
ALDERWOOD_MANOR_CDP = "01185"
ALGER_CDP = "01255"
ALGONA_CITY = "01290"
ALLYN_CDP = "01430"
ALMIRA_TOWN = "01500"
ALTOONA_CDP = "01745"
AMANDA_PARK_CDP = "01780"
AMBOY_CDP = "01850"
AMES_LAKE_CDP = "01920"
ANACORTES_CITY = "01990"
ANATONE_CDP = "02025"
ANDERSON_ISLAND_CDP = "02060"
ARLINGTON_HEIGHTS_CDP = "02620"
ARLINGTON_CITY = "02585"
ARTONDALE_CDP = "02910"
ASHFORD_CDP = "03005"
ASOTIN_CITY = "03075"
AUBURN_CITY = "03180"
BAINBRIDGE_ISLAND_CITY = "03736"
BANGOR_BASE_CDP = "04100"
BANKS_LAKE_SOUTH_CDP = "04125"
BARBERTON_CDP = "04195"
BARING_CDP = "04300"
BARNEYS_JUNCTION_CDP = "04347"
BARSTOW_CDP = "04370"
BASIN_CITY_CDP = "04405"
BATTLE_GROUND_CITY = "04475"
BAY_CENTER_CDP = "04615"
BAY_VIEW_CDP = "04790"
BEACON_HILL_CDP = "04825"
BEAUX_ARTS_VILLAGE_TOWN = "04895"
BELFAIR_CDP = "05140"
BELL_HILL_CDP = "05245"
BELLEVUE_CITY = "05210"
BELLINGHAM_CITY = "05280"
BENTON_CITY_CITY = "05560"
BETHEL_CDP = "05735"
BEVERLY_CDP = "05770"
BICKLETON_CDP = "05980"
BIG_LAKE_CDP = "06050"
BINGEN_CITY = "06085"
BIRCH_BAY_CDP = "06190"
BLACK_DIAMOND_CITY = "06330"
BLAINE_CITY = "06505"
BLYN_CDP = "06855"
BONNEY_LAKE_CITY = "07170"
BOTHELL_EAST_CDP = "07390"
BOTHELL_WEST_CDP = "07395"
BOTHELL_CITY = "07380"
BOULEVARD_PARK_CDP = "07397"
BOW_CDP = "07450"
BOYDS_CDP = "07485"
BRADY_CDP = "07590"
BREMERTON_CITY = "07695"
BREWSTER_CITY = "07835"
BRIDGEPORT_CITY = "07870"
BRIER_CITY = "07940"
BRINNON_CDP = "08080"
BROWNS_POINT_CDP = "08325"
BRUSH_PRAIRIE_CDP = "08465"
BRYANT_CDP = "08500"
BRYN_MAWR_SKYWAY_CDP = "08552"
BUCKLEY_CITY = "08570"
BUCODA_TOWN = "08605"
BUENA_CDP = "08640"
BUNK_FOSS_CDP = "08770"
BURBANK_CDP = "08780"
BURIEN_CITY = "08850"
BURLEY_CDP = "08885"
BURLINGTON_CITY = "08920"
CAMANO_CDP = "09365"
CAMAS_CITY = "09480"
CANTERWOOD_CDP = "09810"
CANYON_CREEK_CDP = "09820"
CARBONADO_TOWN = "09970"
CARLSBORG_CDP = "10075"
CARNATION_CITY = "10215"
CARSON_CDP = "10320"
CASCADE_VALLEY_CDP = "10455"
CASHMERE_CITY = "10495"
CASTLE_ROCK_CITY = "10565"
CATHCART_CDP = "10600"
CATHLAMET_TOWN = "10635"
CAVALERO_CDP = "10645"
CENTERVILLE_CDP = "11090"
CENTRAL_PARK_CDP = "11195"
CENTRALIA_CITY = "11160"
CHAIN_LAKE_CDP = "11325"
CHEHALIS_CITY = "11475"
CHELAN_FALLS_CDP = "11685"
CHELAN_CITY = "11615"
CHENEY_CITY = "11825"
CHERRY_GROVE_CDP = "12015"
CHEWELAH_CITY = "12140"
CHICO_CDP = "12175"
CHINOOK_CDP = "12315"
CLALLAM_BAY_CDP = "12525"
CLARKSTON_HEIGHTS_VINELAND_CDP = "12680"
CLARKSTON_CITY = "12630"
CLAYTON_CDP = "12735"
CLE_ELUM_CITY = "12945"
CLEAR_LAKE_CDP_PIERCE_COUNTY = "12820"
CLEAR_LAKE_CDP_SKAGIT_COUNTY = "12840"
CLEARVIEW_CDP = "12875"
CLIFFDELL_CDP = "13015"
CLINTON_CDP = "13155"
CLOVER_CREEK_CDP = "13215"
CLYDE_HILL_CITY = "13365"
COHASSETT_BEACH_CDP = "13595"
COLFAX_CITY = "13785"
COLLEGE_PLACE_CITY = "13855"
COLTON_TOWN = "13890"
COLVILLE_CITY = "14170"
CONCONULLY_TOWN = "14310"
CONCRETE_TOWN = "14380"
CONNELL_CITY = "14485"
CONWAY_CDP = "14520"
COPALIS_BEACH_CDP = "14660"
COSMOPOLIS_CITY = "14870"
COTTAGE_LAKE_CDP = "14940"
COUGAR_CDP = "15010"
COULEE_CITY_TOWN = "15080"
COULEE_DAM_TOWN = "15115"
COUNTRY_HOMES_CDP = "15150"
COUPEVILLE_TOWN = "15185"
COVINGTON_CITY = "15290"
COWICHE_CDP = "15325"
CRESCENT_BAR_CDP = "15603"
CRESTON_TOWN = "15710"
CROCKER_CDP = "15780"
CURLEW_CDP = "16165"
CURLEW_LAKE_CDP = "16170"
CUSICK_TOWN = "16340"
CUSTER_CDP = "16375"
DALLESPORT_CDP = "16550"
DANVILLE_CDP = "16585"
DARRINGTON_TOWN = "16690"
DASH_POINT_CDP = "16760"
DAVENPORT_CITY = "16795"
DAYTON_CITY = "16970"
DEEP_RIVER_CDP = "17215"
DEER_PARK_CITY = "17320"
DEMING_CDP = "17495"
DES_MOINES_CITY = "17635"
DESERT_AIRE_CDP = "17617"
DISAUTEL_CDP = "17880"
DIXIE_CDP = "18055"
DOLLARS_CORNER_CDP = "18198"
DONALD_CDP = "18265"
DUPONT_CITY = "18965"
DULUTH_CDP = "18775"
DUVALL_CITY = "19035"
EAST_CATHLAMET_CDP = "19290"
EAST_PORT_ORCHARD_CDP = "19770"
EAST_RENTON_HIGHLANDS_CDP = "19857"
EAST_WENATCHEE_CITY = "20155"
EASTMONT_CDP = "19630"
EASTON_CDP = "19700"
EATONVILLE_TOWN = "20260"
EDGEWOOD_CITY = "20645"
EDISON_CDP = "20680"
EDMONDS_CITY = "20750"
ELBE_CDP = "20890"
ELECTRIC_CITY_CITY = "21030"
ELK_PLAIN_CDP = "21205"
ELLENSBURG_CITY = "21240"
ELMA_CITY = "21450"
ELMER_CITY_TOWN = "21485"
ENDICOTT_TOWN = "21730"
ENETAI_CDP = "21800"
ENTIAT_CITY = "22010"
ENUMCLAW_CITY = "22045"
EPHRATA_CITY = "22080"
ERLANDS_POINT_CDP = "22115"
ESCHBACH_CDP = "22150"
ESPERANCE_CDP = "22255"
EVERETT_CITY = "22640"
EVERSON_CITY = "22745"
FAIRCHILD_AFB_CDP = "22955"
FAIRFIELD_TOWN = "22990"
FAIRWOOD_CDP_KING_COUNTY = "23160"
FAIRWOOD_CDP_SPOKANE_COUNTY = "23165"
FALL_CITY_CDP = "23200"
FARMINGTON_TOWN = "23340"
FEDERAL_WAY_CITY = "23515"
FELIDA_CDP = "23550"
FERN_PRAIRIE_CDP = "23690"
FERNDALE_CITY = "23620"
FIFE_HEIGHTS_CDP = "23830"
FIFE_CITY = "23795"
FINLEY_CDP = "23865"
FIRCREST_CITY = "23970"
FIVE_CORNERS_CDP = "24188"
FOBES_HILL_CDP = "24425"
FORDS_PRAIRIE_CDP = "24565"
FORKS_CITY = "24810"
FORT_LEWIS_CDP = "24915"
FOUR_LAKES_CDP = "25300"
FOX_ISLAND_CDP = "25370"
FREDERICKSON_CDP = "25475"
FREELAND_CDP = "25510"
FRIDAY_HARBOR_TOWN = "25615"
GARFIELD_TOWN = "26140"
GARRETT_CDP = "26245"
GENEVA_CDP = "26420"
GEORGE_CITY = "26455"
GIG_HARBOR_CITY = "26735"
GLACIER_CDP = "26875"
GLEED_CDP = "26945"
GLENWOOD_CDP = "27225"
GOLD_BAR_CITY = "27365"
GOLDENDALE_CITY = "27435"
GORST_CDP = "27680"
GRAHAM_CDP = "27785"
GRAND_COULEE_CITY = "27855"
GRAND_MOUND_CDP = "27890"
GRANDVIEW_CITY = "27925"
GRANGER_CITY = "27960"
GRANITE_FALLS_CITY = "27995"
GRAPEVIEW_CDP = "28135"
GRAYLAND_CDP = "28345"
GRAYS_RIVER_CDP = "28450"
GREEN_BLUFF_CDP = "28590"
GREENWATER_CDP = "28695"
HAMILTON_TOWN = "29255"
HANSVILLE_CDP = "29430"
HARRAH_TOWN = "29710"
HARRINGTON_CITY = "29745"
HARTLINE_TOWN = "29920"
HAT_ISLAND_CDP = "30030"
HATTON_TOWN = "30060"
HAZEL_DELL_CDP = "30305"
HERRON_ISLAND_CDP = "30690"
HIGH_BRIDGE_CDP = "30750"
HOBART_CDP = "31495"
HOCKINSON_CDP = "31530"
HOGANS_CORNER_CDP = "31540"
HOME_CDP = "31985"
HOODSPORT_CDP = "32125"
HOQUIAM_CITY = "32300"
HUMPTULIPS_CDP = "32650"
HUNTS_POINT_TOWN = "32755"
ILWACO_CITY = "33000"
INCHELIUM_CDP = "33105"
INDEX_TOWN = "33175"
INDIANOLA_CDP = "33280"
IONE_TOWN = "33560"
ISSAQUAH_CITY = "33805"
JAMESTOWN_CDP = "33890"
KAHLOTUS_CITY = "34575"
KALAMA_CITY = "34645"
KAPOWSIN_CDP = "34890"
KAYAK_POINT_CDP = "34915"
KELLER_CDP = "34960"
KELSO_CITY = "35065"
KENDALL_CDP = "35135"
KENMORE_CITY = "35170"
KENNEWICK_CITY = "35275"
KENT_CITY = "35415"
KETRON_ISLAND_CDP = "35450"
KETTLE_FALLS_CITY = "35485"
KEY_CENTER_CDP = "35555"
KEYPORT_CDP = "35625"
KINGSTON_CDP = "35870"
KIRKLAND_CITY = "35940"
KITSAP_LAKE_CDP = "36010"
KITTITAS_CITY = "36045"
KLICKITAT_CDP = "36115"
KRUPP_TOWN = "36395"
LA_CENTER_CITY = "36710"
LA_CONNER_TOWN = "36780"
LA_GRANDE_CDP = "36920"
LACROSSE_TOWN = "36850"
LACEY_CITY = "36745"
LAKE_BOSWORTH_CDP = "37077"
LAKE_CASSIDY_CDP = "37100"
LAKE_CAVANAUGH_CDP = "37105"
LAKE_FOREST_PARK_CITY = "37270"
LAKE_GOODWIN_CDP = "37287"
LAKE_HOLM_CDP = "37345"
LAKE_KETCHUM_CDP = "37415"
LAKE_MARCEL_STILLWATER_CDP = "37538"
LAKE_MCMURRAY_CDP = "37526"
LAKE_MORTON_BERRYDALE_CDP = "37567"
LAKE_ROESIGER_CDP = "37672"
LAKE_SHORE_CDP = "37830"
LAKE_STEVENS_CITY = "37900"
LAKE_STICKNEY_CDP = "37917"
LAKE_TAPPS_CDP = "37920"
LAKELAND_NORTH_CDP = "37420"
LAKELAND_SOUTH_CDP = "37430"
LAKEVIEW_CDP = "37926"
LAKEWOOD_CITY = "38038"
LAMONT_TOWN = "38215"
LANGLEY_CITY = "38355"
LARCH_WAY_CDP = "38420"
LATAH_TOWN = "38495"
LAURIER_CDP = "38705"
LEAVENWORTH_CITY = "38845"
LEBAM_CDP = "38880"
LEWISVILLE_CDP = "39212"
LEXINGTON_CDP = "39230"
LIBERTY_LAKE_CITY = "39335"
LIND_TOWN = "39510"
LOCHSLOY_CDP = "39860"
LOFALL_CDP = "39930"
LONG_BEACH_CITY = "40070"
LONGBRANCH_CDP = "40105"
LONGVIEW_HEIGHTS_CDP = "40270"
LONGVIEW_CITY = "40245"
LOOMIS_CDP = "40350"
LOON_LAKE_CDP = "40385"
LOWER_ELOCHOMAN_CDP = "40570"
LYLE_CDP = "40735"
LYMAN_TOWN = "40770"
LYNDEN_CITY = "40805"
LYNNWOOD_CITY = "40840"
MABTON_CITY = "40980"
MACHIAS_CDP = "41470"
MALDEN_TOWN = "42275"
MALO_CDP = "42310"
MALONE_CDP = "42345"
MALOTT_CDP = "42380"
MALTBY_CDP = "42415"
MANCHESTER_CDP = "42450"
MANSFIELD_TOWN = "42800"
MANSON_CDP = "42835"
MAPLE_FALLS_CDP = "43010"
MAPLE_HEIGHTS_LAKE_DESIRE_CDP = "43062"
MAPLE_VALLEY_CITY = "43150"
MAPLEWOOD_CDP = "43255"
MARBLEMOUNT_CDP = "43325"
MARCUS_TOWN = "43395"
MARIETTA_ALDERWOOD_CDP = "43491"
MARINE_VIEW_CDP = "43552"
MARKHAM_CDP = "43640"
MARROWSTONE_CDP = "43762"
MARTHA_LAKE_CDP = "43815"
MARYHILL_CDP = "43885"
MARYSVILLE_CITY = "43955"
MATTAWA_CITY = "44165"
MAY_CREEK_CDP = "44260"
MCCHORD_AFB_CDP = "41155"
MCCLEARY_CITY = "41225"
MCKENNA_CDP = "41645"
MCMILLIN_CDP = "41785"
MEAD_CDP = "44480"
MEADOW_GLADE_CDP = "44620"
MEADOWDALE_CDP = "44585"
MEDICAL_LAKE_CITY = "44690"
MEDINA_CITY = "44725"
MERCER_ISLAND_CITY = "45005"
MESA_CITY = "45180"
METALINE_FALLS_TOWN = "45320"
METALINE_TOWN = "45285"
METHOW_CDP = "45355"
MIDLAND_CDP = "45495"
MILL_CREEK_EAST_CDP = "45870"
MILL_CREEK_CITY = "45865"
MILLWOOD_CITY = "45985"
MILTON_CITY = "46020"
MINERAL_CDP = "46090"
MINNEHAHA_CDP = "46125"
MIRRORMONT_CDP = "46215"
MOCLIPS_CDP = "46405"
MONROE_NORTH_CDP = "46725"
MONROE_CITY = "46685"
MONTESANO_CITY = "46895"
MORTON_CITY = "47175"
MOSES_LAKE_NORTH_CDP = "47280"
MOSES_LAKE_CITY = "47245"
MOSSYROCK_CITY = "47315"
MOUNT_VERNON_CITY = "47560"
MOUNT_VISTA_CDP = "47630"
MOUNTLAKE_TERRACE_CITY = "47490"
MOXEE_CITY = "47665"
MUKILTEO_CITY = "47735"
NACHES_TOWN = "47805"
NAPAVINE_CITY = "47980"
NASELLE_CDP = "48015"
NAVY_YARD_CITY_CDP = "48225"
NEAH_BAY_CDP = "48295"
NEILTON_CDP = "48330"
NESPELEM_COMMUNITY_CDP = "48550"
NESPELEM_TOWN = "48540"
NEWCASTLE_CITY = "48645"
NEWPORT_CITY = "48820"
NILE_CDP = "49030"
NISQUALLY_INDIAN_COMMUNITY_CDP = "49193"
NOOKSACK_CITY = "49275"
NORMANDY_PARK_CITY = "49415"
NORTH_BEND_CITY = "49485"
NORTH_BONNEVILLE_CITY = "49555"
NORTH_FORT_LEWIS_CDP = "49765"
NORTH_LYNNWOOD_CDP = "49940"
NORTH_OMAK_CDP = "50007"
NORTH_PUYALLUP_CDP = "50115"
NORTH_SULTAN_CDP = "50183"
NORTH_YELM_CDP = "50210"
NORTHPORT_TOWN = "50045"
NORTHWEST_STANWOOD_CDP = "50195"
OAK_HARBOR_CITY = "50360"
OAKESDALE_TOWN = "50325"
OAKVILLE_CITY = "50430"
OCEAN_CITY_CDP = "50500"
OCEAN_PARK_CDP = "50535"
OCEAN_SHORES_CITY = "50570"
OCOSTA_CDP = "50675"
ODESSA_TOWN = "50745"
OKANOGAN_CITY = "50920"
OLYMPIA_CITY = "51300"
OMAK_CITY = "51340"
ONALASKA_CDP = "51410"
ORCHARDS_CDP = "51795"
ORIENT_CDP = "51830"
OROVILLE_CITY = "51970"
ORTING_CITY = "52005"
OSO_CDP = "52110"
OTHELLO_CITY = "52215"
OTIS_ORCHARDS_EAST_FARMS_CDP = "52267"
OUTLOOK_CDP = "52285"
OYEHUT_CDP = "52378"
PACIFIC_BEACH_CDP = "52530"
PACIFIC_CITY = "52495"
PACKWOOD_CDP = "52600"
PALOUSE_CITY = "52950"
PARKER_CDP = "53265"
PARKLAND_CDP = "53335"
PARKWOOD_CDP = "53440"
PASCO_CITY = "53545"
PATAHA_CDP = "53660"
PATEROS_CITY = "53720"
PE_ELL_TOWN = "53930"
PEACEFUL_VALLEY_CDP = "53800"
PICNIC_POINT_CDP = "54213"
PINE_GROVE_CDP = "54405"
POINT_ROBERTS_CDP = "55015"
POMEROY_CITY = "55120"
PORT_ANGELES_EAST_CDP = "55400"
PORT_ANGELES_CITY = "55365"
PORT_GAMBLE_TRIBAL_COMMUNITY_CDP = "55612"
PORT_HADLOCK_IRONDALE_CDP = "55620"
PORT_LUDLOW_CDP = "55645"
PORT_ORCHARD_CITY = "55785"
PORT_TOWNSEND_CITY = "55855"
PORTER_CDP = "55540"
POULSBO_CITY = "55995"
PRAIRIE_HEIGHTS_CDP = "56150"
PRAIRIE_RIDGE_CDP = "56170"
PRESCOTT_CITY = "56240"
PROSSER_CITY = "56450"
PUGET_ISLAND_CDP = "56555"
PULLMAN_CITY = "56625"
PURDY_CDP = "56660"
PUYALLUP_CITY = "56695"
QUEETS_CDP = "56905"
QUI_NAI_ELT_VILLAGE_CDP = "57030"
QUILCENE_CDP = "56975"
QUINCY_CITY = "57115"
RAFT_ISLAND_CDP = "57140"
RAINIER_CITY = "57220"
RAVENSDALE_CDP = "57395"
RAYMOND_CITY = "57430"
REARDAN_TOWN = "57465"
REDMOND_CITY = "57535"
RENTON_CITY = "57745"
REPUBLIC_CITY = "57850"
RICHLAND_CITY = "58235"
RIDGEFIELD_CITY = "58410"
RITZVILLE_CITY = "58725"
RIVER_ROAD_CDP = "58777"
RIVERBEND_CDP = "58742"
RIVERPOINT_CDP = "58768"
RIVERSIDE_TOWN = "58795"
ROCHE_HARBOR_CDP = "59075"
ROCHESTER_CDP = "59110"
ROCK_ISLAND_CITY = "59180"
ROCKFORD_TOWN = "59145"
ROCKPORT_CDP = "59250"
ROCKY_POINT_CDP = "59390"
RONALD_CDP = "59635"
ROOSEVELT_CDP = "59705"
ROSALIA_TOWN = "59775"
ROSBURG_CDP = "59845"
ROSEDALE_CDP = "59880"
ROSLYN_CITY = "60055"
ROY_CITY = "60160"
ROYAL_CITY_CITY = "60230"
RUSTON_CITY = "60510"
RYDERWOOD_CDP = "60580"
SALMON_CREEK_CDP = "61000"
SAMMAMISH_CITY = "61115"
SANTIAGO_CDP = "61235"
SATSOP_CDP = "61350"
SCHWANA_CDP = "61925"
SEATAC_CITY = "62288"
SEABECK_CDP = "62120"
SEATTLE_CITY = "63000"
SEDRO_WOOLLEY_CITY = "63210"
SEKIU_CDP = "63245"
SELAH_CITY = "63280"
SEQUIM_CITY = "63385"
SHADOW_LAKE_CDP = "63545"
SHELTON_CITY = "63735"
SHORELINE_CITY = "63960"
SILVANA_CDP = "64190"
SILVER_FIRS_CDP = "64380"
SILVERDALE_CDP = "64365"
SISCO_HEIGHTS_CDP = "64610"
SKAMOKAWA_VALLEY_CDP = "64755"
SKOKOMISH_CDP = "64775"
SKYKOMISH_TOWN = "64855"
SNOHOMISH_CITY = "65170"
SNOQUALMIE_PASS_CDP = "65275"
SNOQUALMIE_CITY = "65205"
SOAP_LAKE_CITY = "65345"
SOUTH_BEND_CITY = "65625"
SOUTH_CLE_ELUM_TOWN = "65765"
SOUTH_CREEK_CDP = "65810"
SOUTH_HILL_CDP = "65922"
SOUTH_PRAIRIE_TOWN = "66045"
SOUTH_WENATCHEE_CDP = "66185"
SOUTHWORTH_CDP = "66220"
SPANAWAY_CDP = "66255"
SPANGLE_CITY = "66290"
SPOKANE_VALLEY_CITY = "67167"
SPOKANE_CITY = "67000"
SPRAGUE_CITY = "67175"
SPRINGDALE_TOWN = "67210"
ST_JOHN_TOWN = "60860"
STANSBERRY_LAKE_CDP = "67435"
STANWOOD_CITY = "67455"
STARBUCK_TOWN = "67490"
STARTUP_CDP = "67595"
STEILACOOM_TOWN = "67770"
STEPTOE_CDP = "67805"
STEVENSON_CITY = "67875"
SUDDEN_VALLEY_CDP = "68200"
SULTAN_CITY = "68260"
SUMAS_CITY = "68330"
SUMMIT_CDP = "68365"
SUMMIT_VIEW_CDP = "68410"
SUMMITVIEW_CDP = "68417"
SUMNER_CITY = "68435"
SUNCREST_CDP = "68460"
SUNDAY_LAKE_CDP = "68480"
SUNLAND_ESTATES_CDP = "68557"
SUNNYSIDE_CITY = "68750"
SUNNYSLOPE_CDP = "68785"
SUQUAMISH_CDP = "69170"
SWEDE_HEAVEN_CDP = "69280"
TACOMA_CITY = "70000"
TAHOLAH_CDP = "70175"
TAMPICO_CDP = "70245"
TANGLEWILDE_CDP = "70280"
TEHALEH_CDP = "70543"
TEKOA_CITY = "70560"
TENINO_CITY = "70630"
TERRACE_HEIGHTS_CDP = "70805"
THORP_CDP = "71225"
THREE_LAKES_CDP = "71330"
TIETON_CITY = "71400"
TOKELAND_CDP = "71680"
TOLEDO_CITY = "71785"
TONASKET_CITY = "71890"
TOPPENISH_CITY = "71960"
TORBOY_CDP = "71995"
TOUCHET_CDP = "72030"
TOWN_AND_COUNTRY_CDP = "72170"
TRACYTON_CDP = "72205"
TROUT_LAKE_CDP = "72450"
TUKWILA_CITY = "72625"
TUMWATER_CITY = "72905"
TWIN_LAKES_CDP = "73065"
TWISP_TOWN = "73080"
UNION_CDP = "73255"
UNION_GAP_CITY = "73290"
UNION_HILL_NOVELTY_HILL_CDP = "73307"
UNIONTOWN_TOWN = "73360"
UNIVERSITY_PLACE_CITY = "73465"
UPPER_ELOCHOMAN_CDP = "73580"
VADER_CITY = "73780"
VALLEY_CDP = "73885"
VANCOUVER_CITY = "74060"
VANTAGE_CDP = "74200"
VASHON_CDP = "74305"
VAUGHN_CDP = "74445"
VENERSBORG_CDP = "74585"
VERLOT_CDP = "74760"
WAITSBURG_CITY = "75565"
WALLA_WALLA_EAST_CDP = "75810"
WALLA_WALLA_CITY = "75775"
WALLER_CDP = "75905"
WALLULA_CDP = "75985"
WAPATO_CITY = "76125"
WARDEN_CITY = "76160"
WARM_BEACH_CDP = "76195"
WASHOUGAL_CITY = "76405"
WASHTUCNA_TOWN = "76440"
WATERVILLE_TOWN = "76510"
WAUNA_CDP = "76615"
WAVERLY_TOWN = "76720"
WENATCHEE_CITY = "77105"
WEST_CLARKSTON_HIGHLAND_CDP = "77297"
WEST_PASCO_CDP = "77612"
WEST_RICHLAND_CITY = "77665"
WESTPORT_CITY = "77630"
WHEELER_CDP = "78050"
WHIDBEY_ISLAND_STATION_CDP = "78155"
WHITE_CENTER_CDP = "78225"
WHITE_SALMON_CITY = "78330"
WHITE_SWAN_CDP = "78365"
WILBUR_TOWN = "78680"
WILDERNESS_RIM_CDP = "78780"
WILKESON_TOWN = "78925"
WILLAPA_CDP = "78995"
WILSON_CREEK_TOWN = "79135"
WINLOCK_CITY = "79275"
WINTHROP_TOWN = "79380"
WISHRAM_CDP = "79485"
WOLLOCHET_CDP = "79555"
WOODINVILLE_CITY = "79590"
WOODLAND_CITY = "79625"
WOODS_CREEK_CDP = "79825"
WOODWAY_CITY = "79835"
YACOLT_TOWN = "79975"
YAKIMA_CITY = "80010"
YARROW_POINT_TOWN = "80150"
YELM_CITY = "80220"
ZILLAH_CITY = "80500"



# Copyright (c) 2023 Darren Erik Vengroff
"""Counties in Texas."""


ANDERSON = "001"
ANDREWS = "003"
ANGELINA = "005"
ARANSAS = "007"
ARCHER = "009"
ARMSTRONG = "011"
ATASCOSA = "013"
AUSTIN = "015"
BAILEY = "017"
BANDERA = "019"
BASTROP = "021"
BAYLOR = "023"
BEE = "025"
BELL = "027"
BEXAR = "029"
BLANCO = "031"
BORDEN = "033"
BOSQUE = "035"
BOWIE = "037"
BRAZORIA = "039"
BRAZOS = "041"
BREWSTER = "043"
BRISCOE = "045"
BROOKS = "047"
BROWN = "049"
BURLESON = "051"
BURNET = "053"
CALDWELL = "055"
CALHOUN = "057"
CALLAHAN = "059"
CAMERON = "061"
CAMP = "063"
CARSON = "065"
CASS = "067"
CASTRO = "069"
CHAMBERS = "071"
CHEROKEE = "073"
CHILDRESS = "075"
CLAY = "077"
COCHRAN = "079"
COKE = "081"
COLEMAN = "083"
COLLIN = "085"
COLLINGSWORTH = "087"
COLORADO = "089"
COMAL = "091"
COMANCHE = "093"
CONCHO = "095"
COOKE = "097"
CORYELL = "099"
COTTLE = "101"
CRANE = "103"
CROCKETT = "105"
CROSBY = "107"
CULBERSON = "109"
DALLAM = "111"
DALLAS = "113"
DAWSON = "115"
DEWITT = "123"
DEAF_SMITH = "117"
DELTA = "119"
DENTON = "121"
DICKENS = "125"
DIMMIT = "127"
DONLEY = "129"
DUVAL = "131"
EASTLAND = "133"
ECTOR = "135"
EDWARDS = "137"
EL_PASO = "141"
ELLIS = "139"
ERATH = "143"
FALLS = "145"
FANNIN = "147"
FAYETTE = "149"
FISHER = "151"
FLOYD = "153"
FOARD = "155"
FORT_BEND = "157"
FRANKLIN = "159"
FREESTONE = "161"
FRIO = "163"
GAINES = "165"
GALVESTON = "167"
GARZA = "169"
GILLESPIE = "171"
GLASSCOCK = "173"
GOLIAD = "175"
GONZALES = "177"
GRAY = "179"
GRAYSON = "181"
GREGG = "183"
GRIMES = "185"
GUADALUPE = "187"
HALE = "189"
HALL = "191"
HAMILTON = "193"
HANSFORD = "195"
HARDEMAN = "197"
HARDIN = "199"
HARRIS = "201"
HARRISON = "203"
HARTLEY = "205"
HASKELL = "207"
HAYS = "209"
HEMPHILL = "211"
HENDERSON = "213"
HIDALGO = "215"
HILL = "217"
HOCKLEY = "219"
HOOD = "221"
HOPKINS = "223"
HOUSTON = "225"
HOWARD = "227"
HUDSPETH = "229"
HUNT = "231"
HUTCHINSON = "233"
IRION = "235"
JACK = "237"
JACKSON = "239"
JASPER = "241"
JEFF_DAVIS = "243"
JEFFERSON = "245"
JIM_HOGG = "247"
JIM_WELLS = "249"
JOHNSON = "251"
JONES = "253"
KARNES = "255"
KAUFMAN = "257"
KENDALL = "259"
KENEDY = "261"
KENT = "263"
KERR = "265"
KIMBLE = "267"
KING = "269"
KINNEY = "271"
KLEBERG = "273"
KNOX = "275"
LA_SALLE = "283"
LAMAR = "277"
LAMB = "279"
LAMPASAS = "281"
LAVACA = "285"
LEE = "287"
LEON = "289"
LIBERTY = "291"
LIMESTONE = "293"
LIPSCOMB = "295"
LIVE_OAK = "297"
LLANO = "299"
LOVING = "301"
LUBBOCK = "303"
LYNN = "305"
MADISON = "313"
MARION = "315"
MARTIN = "317"
MASON = "319"
MATAGORDA = "321"
MAVERICK = "323"
MCCULLOCH = "307"
MCLENNAN = "309"
MCMULLEN = "311"
MEDINA = "325"
MENARD = "327"
MIDLAND = "329"
MILAM = "331"
MILLS = "333"
MITCHELL = "335"
MONTAGUE = "337"
MONTGOMERY = "339"
MOORE = "341"
MORRIS = "343"
MOTLEY = "345"
NACOGDOCHES = "347"
NAVARRO = "349"
NEWTON = "351"
NOLAN = "353"
NUECES = "355"
OCHILTREE = "357"
OLDHAM = "359"
ORANGE = "361"
PALO_PINTO = "363"
PANOLA = "365"
PARKER = "367"
PARMER = "369"
PECOS = "371"
POLK = "373"
POTTER = "375"
PRESIDIO = "377"
RAINS = "379"
RANDALL = "381"
REAGAN = "383"
REAL = "385"
RED_RIVER = "387"
REEVES = "389"
REFUGIO = "391"
ROBERTS = "393"
ROBERTSON = "395"
ROCKWALL = "397"
RUNNELS = "399"
RUSK = "401"
SABINE = "403"
SAN_AUGUSTINE = "405"
SAN_JACINTO = "407"
SAN_PATRICIO = "409"
SAN_SABA = "411"
SCHLEICHER = "413"
SCURRY = "415"
SHACKELFORD = "417"
SHELBY = "419"
SHERMAN = "421"
SMITH = "423"
SOMERVELL = "425"
STARR = "427"
STEPHENS = "429"
STERLING = "431"
STONEWALL = "433"
SUTTON = "435"
SWISHER = "437"
TARRANT = "439"
TAYLOR = "441"
TERRELL = "443"
TERRY = "445"
THROCKMORTON = "447"
TITUS = "449"
TOM_GREEN = "451"
TRAVIS = "453"
TRINITY = "455"
TYLER = "457"
UPSHUR = "459"
UPTON = "461"
UVALDE = "463"
VAL_VERDE = "465"
VAN_ZANDT = "467"
VICTORIA = "469"
WALKER = "471"
WALLER = "473"
WARD = "475"
WASHINGTON = "477"
WEBB = "479"
WHARTON = "481"
WHEELER = "483"
WICHITA = "485"
WILBARGER = "487"
WILLACY = "489"
WILLIAMSON = "491"
WILSON = "493"
WINKLER = "495"
WISE = "497"
WOOD = "499"
YOAKUM = "501"
YOUNG = "503"
ZAPATA = "505"
ZAVALA = "507"



# Copyright (c) 2023 Darren Erik Vengroff
"""Counties in Oregon."""


BAKER = "001"
BENTON = "003"
CLACKAMAS = "005"
CLATSOP = "007"
COLUMBIA = "009"
COOS = "011"
CROOK = "013"
CURRY = "015"
DESCHUTES = "017"
DOUGLAS = "019"
GILLIAM = "021"
GRANT = "023"
HARNEY = "025"
HOOD_RIVER = "027"
JACKSON = "029"
JEFFERSON = "031"
JOSEPHINE = "033"
KLAMATH = "035"
LAKE = "037"
LANE = "039"
LINCOLN = "041"
LINN = "043"
MALHEUR = "045"
MARION = "047"
MORROW = "049"
MULTNOMAH = "051"
POLK = "053"
SHERMAN = "055"
TILLAMOOK = "057"
UMATILLA = "059"
UNION = "061"
WALLOWA = "063"
WASCO = "065"
WASHINGTON = "067"
WHEELER = "069"
YAMHILL = "071"



# Copyright (c) 2023 Darren Erik Vengroff
"""Counties in Mississippi."""


ADAMS = "001"
ALCORN = "003"
AMITE = "005"
ATTALA = "007"
BENTON = "009"
BOLIVAR = "011"
CALHOUN = "013"
CARROLL = "015"
CHICKASAW = "017"
CHOCTAW = "019"
CLAIBORNE = "021"
CLARKE = "023"
CLAY = "025"
COAHOMA = "027"
COPIAH = "029"
COVINGTON = "031"
DESOTO = "033"
FORREST = "035"
FRANKLIN = "037"
GEORGE = "039"
GREENE = "041"
GRENADA = "043"
HANCOCK = "045"
HARRISON = "047"
HINDS = "049"
HOLMES = "051"
HUMPHREYS = "053"
ISSAQUENA = "055"
ITAWAMBA = "057"
JACKSON = "059"
JASPER = "061"
JEFFERSON = "063"
JEFFERSON_DAVIS = "065"
JONES = "067"
KEMPER = "069"
LAFAYETTE = "071"
LAMAR = "073"
LAUDERDALE = "075"
LAWRENCE = "077"
LEAKE = "079"
LEE = "081"
LEFLORE = "083"
LINCOLN = "085"
LOWNDES = "087"
MADISON = "089"
MARION = "091"
MARSHALL = "093"
MONROE = "095"
MONTGOMERY = "097"
NESHOBA = "099"
NEWTON = "101"
NOXUBEE = "103"
OKTIBBEHA = "105"
PANOLA = "107"
PEARL_RIVER = "109"
PERRY = "111"
PIKE = "113"
PONTOTOC = "115"
PRENTISS = "117"
QUITMAN = "119"
RANKIN = "121"
SCOTT = "123"
SHARKEY = "125"
SIMPSON = "127"
SMITH = "129"
STONE = "131"
SUNFLOWER = "133"
TALLAHATCHIE = "135"
TATE = "137"
TIPPAH = "139"
TISHOMINGO = "141"
TUNICA = "143"
UNION = "145"
WALTHALL = "147"
WARREN = "149"
WASHINGTON = "151"
WAYNE = "153"
WEBSTER = "155"
WILKINSON = "157"
WINSTON = "159"
YALOBUSHA = "161"
YAZOO = "163"



# Copyright (c) 2022 Darren Erik Vengroff
"""
Abstract base class for variable sources.

The two concrete implementations are a mock
version for testing and a version that loads
from the U.S. Census API.
"""

from abc import ABC, abstractmethod
from typing import Any, Dict, List, Literal, Optional, Union

VintageType = Union[int, Literal["timeseries"]]
"""
The type we use to specify the vintage of a dataset.

Most datasets are organized by year, so we pass an integer
year like 2020. But some datasets are timeseries that cover
multiple years, so we specify the literal value "timeseries".
"""


class VariableSource(ABC):
    """
    A source of variables, typically used behind a :py:class:`~VariableCache`.

    The purpose of this class is to get variable and group information
    from a source, typically a remote API call to the US Census API.
    Another use case is to enable mocking for testing the rest of the
    :py:class:`~VariableCache` functionality, which is a superset of
    what this class does.
    """

    @abstractmethod
    def get(
        self,
        dataset: str,
        year: int,
        name: str,
    ) -> Dict[str, Any]:
        """
        Get information on a variable for a given dataset in a given year.

        The return value is a dictionary with the following fields:

        .. list-table:: Title
            :widths: 25 75
            :header-rows: 0

            * - `"name"`
              - The name of the variable.
            * - '"label"`
              - A description of the variable. Within groups, hierarchies of
                variables are represented by seperating levels with `"!!"`.
            * - `"concept"`
              - The concept this variable and others in the group represent.
            * - `"group"`
              - The group the variable belongs to. To query an entire group,
                use the :py:meth:`~get_group` method.
            * - `"limit"`
              -
            * - `"attributes"`
              - A comma-separated list of variables that are attributes of this
                one.

        This dictionary is very much like the JSON returned from US Census
        API URLs like
        https://api.census.gov/data/2020/acs/acs5/variables/B03001_001E.json

        Parameters
        ----------
        dataset
            The census dataset, for example `dec/acs5` for ACS5 data
            (https://www.census.gov/data/developers/data-sets/acs-5year.html and
            https://api.census.gov/data/2020/acs/acs5.html)
            or `dec/pl` for redistricting data
            (https://www.census.gov/programs-surveys/decennial-census/about/rdo.html and
            https://api.census.gov/data/2020/dec/pl.html)
        year
            The year
        name
            The name of the variable to get information about. For example,
            `B03002_001E` is a variable from the ACS5 data set that represents
            total population in a geographic area.

        Returns
        -------
            A dictionary of information about the variable.
        """
        raise NotImplementedError("Abstract method.")

    @abstractmethod
    def get_group(
        self,
        dataset: str,
        year: int,
        name: str,
    ) -> Dict[str, Dict]:
        """
        Get information on a group of variables for a given dataset in a given year.

        The return value is a dictionary that is very much like the JSON returned
        from US Census API URLs like
        https://api.census.gov/data/2020/acs/acs5/groups/B03002.json

        See :py:meth:`~VariableSource.get` for more details.

        Parameters
        ----------
        dataset
            The census dataset, for example `dec/acs5` for ACS5 data
            (https://www.census.gov/data/developers/data-sets/acs-5year.html and
            https://api.census.gov/data/2020/acs/acs5.html)
            or `dec/pl` for redistricting data
            (https://www.census.gov/programs-surveys/decennial-census/about/rdo.html and
            https://api.census.gov/data/2020/dec/pl.html)
        year
            The year
        name
            The name of the group to get information about. For example,
            `B03002` is a group from the ACS5 data set that contains
            variables that represent the population of various racial and
            ethnic groups in a geographic area.

        Returns
        -------
            A dictionary with a single key `"variables"`. The value
            associated with that key is a dictionary that maps from the
            names of variables in the group to dictionaries of attributes
            of the variable, in the same form as that returned for individual
            variables by the method :py:meth:`~VariableSource.get`.
        """
        raise NotImplementedError("Abstract method.")

    @abstractmethod
    def get_all_groups(self, dataset: str, year: int) -> Dict[str, List]:
        """
        Get information on a group of variables for a given dataset in a given year.

        The return value is a dictionary that is very much like the JSON returned
        from US Census API URLs like
        https://api.census.gov/data/2020/acs/acs5/groups.json

        See :py:meth:`~VariableSource.get_all_groups` for more details.

        dataset
            The census dataset, for example `dec/acs5` for ACS5 data
            (https://www.census.gov/data/developers/data-sets/acs-5year.html and
            https://api.census.gov/data/2020/acs/acs5.html)
            or `dec/pl` for redistricting data
            (https://www.census.gov/programs-surveys/decennial-census/about/rdo.html and
            https://api.census.gov/data/2020/dec/pl.html)
        year
            The year

        Returns
        -------
            A dictionary with a single key `"groups"`. The value
            associated with that key is a dictionary that maps from the
            names of groups to dictionaries of attributes
            of each group.
        """
        raise NotImplementedError("Abstract method.")

    @abstractmethod
    def get_datasets(self, year: Optional[int]) -> Dict[str, Any]:
        """
        Get descriptions of all the datasets available for a given year.

        Parameters
        ----------
        year
            The year. If `None`, get all datasets for all years.

        Returns
        -------
            A dictionary with a key "datasets". The value associated
            with that key is a dictionary that maps from the names
            of data sets to dictionaries of attributes of each data
            set.
        """
        raise NotImplementedError("Abstract method.")



# Copyright (c) 2022-2023 Darren Erik Vengroff
"""
Tools for downloading and analyzing U.S. Census data.

Includes plotting maps and computing diversity and integration metrics.

Both Python and CLI interfaces are available.
"""
from .impl.exceptions import CensusApiException
from usingversion import getattr_with_version

__all__ = ("CensusApiException",)

# Generate a version number.
__getattr__ = getattr_with_version("impactchart", __file__, __name__)



# Copyright (c) 2023 Darren Erik Vengroff
"""Counties in Maryland."""


ALLEGANY = "001"
ANNE_ARUNDEL = "003"
BALTIMORE = "005"
BALTIMORE_CITY = "510"
CALVERT = "009"
CAROLINE = "011"
CARROLL = "013"
CECIL = "015"
CHARLES = "017"
DORCHESTER = "019"
FREDERICK = "021"
GARRETT = "023"
HARFORD = "025"
HOWARD = "027"
KENT = "029"
MONTGOMERY = "031"
PRINCE_GEORGES = "033"
QUEEN_ANNES = "035"
SOMERSET = "039"
ST_MARYS = "037"
TALBOT = "041"
WASHINGTON = "043"
WICOMICO = "045"
WORCESTER = "047"



# Copyright (c) 2023 Darren Erik Vengroff
"""Counties in New Jersey."""


ATLANTIC = "001"
BERGEN = "003"
BURLINGTON = "005"
CAMDEN = "007"
CAPE_MAY = "009"
CUMBERLAND = "011"
ESSEX = "013"
GLOUCESTER = "015"
HUDSON = "017"
HUNTERDON = "019"
MERCER = "021"
MIDDLESEX = "023"
MONMOUTH = "025"
MORRIS = "027"
OCEAN = "029"
PASSAIC = "031"
SALEM = "033"
SOMERSET = "035"
SUSSEX = "037"
UNION = "039"
WARREN = "041"



# Copyright (c) 2023 Darren Erik Vengroff
"""Places in Georgia."""


ABBEVILLE_CITY = "00184"
ACWORTH_CITY = "00408"
ADAIRSVILLE_CITY = "00436"
ADEL_CITY = "00576"
ADRIAN_CITY = "00660"
AILEY_CITY = "00828"
ALAMO_TOWN = "00996"
ALAPAHA_TOWN = "01024"
ALBANY_CITY = "01052"
ALDORA_TOWN = "01248"
ALLENHURST_CITY = "01472"
ALLENTOWN_CITY = "01500"
ALMA_CITY = "01612"
ALPHARETTA_CITY = "01696"
ALSTON_TOWN = "01808"
ALTO_TOWN = "01948"
AMBROSE_CITY = "02060"
AMERICUS_CITY = "02116"
ANDERSONVILLE_CITY = "02256"
ANTIOCH_CDP = "02364"
APPLING_CDP = "02536"
ARABI_TOWN = "02564"
ARAGON_CITY = "02592"
ARCADE_CITY = "02648"
ARGYLE_TOWN = "02844"
ARLINGTON_CITY = "02928"
ARNOLDSVILLE_CITY = "03124"
ASHBURN_CITY = "03236"
ATHENS_CLARKE_COUNTY_UNIFIED_GOVERNMENT_BALANCE = "03440"
ATLANTA_CITY = "04000"
ATTAPULGUS_CITY = "04084"
AUBURN_CITY = "04140"
AUGUSTA_RICHMOND_COUNTY_CONSOLIDATED_GOVERNMENT_BALANCE = "04204"
AUSTELL_CITY = "04252"
AVALON_TOWN = "04392"
AVERA_CITY = "04448"
AVONDALE_ESTATES_CITY = "04644"
AXSON_CDP = "04700"
BACONTON_CITY = "04840"
BAINBRIDGE_CITY = "04896"
BALDWIN_CITY = "04980"
BALL_GROUND_CITY = "05036"
BARNESVILLE_CITY = "05344"
BARTOW_TOWN = "05680"
BARWICK_CITY = "05708"
BAXLEY_CITY = "06016"
BELLVILLE_CITY = "06688"
BELVEDERE_PARK_CDP = "06884"
BEMISS_CDP = "06968"
BERKELEY_LAKE_CITY = "07248"
BERLIN_CITY = "07304"
BERRY_COLLEGE_CDP = "07380"
BETHLEHEM_TOWN = "07612"
BETWEEN_TOWN = "07640"
BISHOP_TOWN = "08200"
BLACKSHEAR_CITY = "08284"
BLAIRSVILLE_CITY = "08480"
BLAKELY_CITY = "08536"
BLOOMINGDALE_CITY = "08844"
BLUE_RIDGE_CITY = "08928"
BLUFFTON_TOWN = "08956"
BLYTHE_CITY = "09040"
BOGART_TOWN = "09068"
BOLINGBROKE_CDP = "09152"
BONANZA_CDP = "09272"
BOSTON_CITY = "09460"
BOSTWICK_CITY = "09488"
BOWDON_CITY = "09544"
BOWERSVILLE_TOWN = "09656"
BOWMAN_CITY = "09712"
BOX_SPRINGS_CDP = "09768"
BOYKIN_CDP = "09852"
BRASELTON_TOWN = "10076"
BRASWELL_TOWN = "10104"
BREMEN_CITY = "10132"
BRINSON_TOWN = "10608"
BRISTOL_CDP = "10664"
BRONWOOD_TOWN = "10860"
BROOKHAVEN_CITY = "10944"
BROOKLET_CITY = "11000"
BROOKS_TOWN = "11056"
BROXTON_CITY = "11504"
BRUNSWICK_CITY = "11560"
BUCHANAN_CITY = "11616"
BUCKHEAD_CDP = "11626"
BUCKHEAD_TOWN = "11672"
BUENA_VISTA_CITY = "11728"
BUFORD_CITY = "11784"
BUTLER_CITY = "12120"
BYROMVILLE_TOWN = "12232"
BYRON_CITY = "12260"
CADWELL_TOWN = "12344"
CAIRO_CITY = "12400"
CALHOUN_CITY = "12456"
CALVARY_CDP = "12484"
CAMAK_TOWN = "12512"
CAMILLA_CITY = "12624"
CANDLER_MCAFEE_CDP = "12834"
CANON_CITY = "12932"
CANOOCHEE_CDP = "12960"
CANTON_CITY = "12988"
CARL_TOWN = "13184"
CARLTON_CITY = "13212"
CARNESVILLE_CITY = "13352"
CARROLLTON_CITY = "13492"
CARTERSVILLE_CITY = "13688"
CATAULA_CDP = "14024"
CAVE_SPRING_CITY = "14108"
CECIL_CITY = "14192"
CEDAR_SPRINGS_CDP = "14472"
CEDARTOWN_CITY = "14500"
CENTERVILLE_CITY = "14920"
CENTRALHATCHEE_TOWN = "15026"
CHAMBLEE_CITY = "15172"
CHATSWORTH_CITY = "15508"
CHATTAHOOCHEE_HILLS_CITY = "15552"
CHATTANOOGA_VALLEY_CDP = "15585"
CHAUNCEY_CITY = "15648"
CHERRY_LOG_CDP = "15844"
CHESTER_TOWN = "15900"
CHICKAMAUGA_CITY = "15984"
CLARKESVILLE_CITY = "16460"
CLARKSTON_CITY = "16544"
CLAXTON_CITY = "16600"
CLAYTON_CITY = "16656"
CLERMONT_TOWN = "16796"
CLEVELAND_CITY = "16824"
CLIMAX_CITY = "16908"
CLYATTVILLE_CDP = "17132"
COBBTOWN_CITY = "17272"
COCHRAN_CITY = "17328"
COGDELL_CDP = "17468"
COHUTTA_CITY = "17496"
COLBERT_CITY = "17552"
COLEMAN_CDP = "17580"
COLLEGE_PARK_CITY = "17776"
COLLINS_CITY = "17832"
COLQUITT_CITY = "18000"
COLUMBUS_CITY = "19000"
COMER_CITY = "19084"
COMMERCE_CITY = "19112"
CONCORD_CITY = "19168"
CONLEY_CDP = "19280"
CONYERS_CITY = "19336"
COOLIDGE_CITY = "19392"
CORDELE_CITY = "19616"
CORNELIA_CITY = "19728"
COUNTRY_CLUB_ESTATES_CDP = "19830"
COVINGTON_CITY = "20064"
CRAWFORD_CITY = "20288"
CRAWFORDVILLE_CITY = "20316"
CRESCENT_CDP = "20372"
CROOKED_CREEK_CDP = "20550"
CULLODEN_CITY = "20848"
CUMMING_CITY = "20932"
CUSSETA_CHATTAHOOCHEE_COUNTY_UNIFIED_GOVERNMENT = "21017"
CUTHBERT_CITY = "21072"
DACULA_CITY = "21184"
DAHLONEGA_CITY = "21240"
DAISY_CITY = "21268"
DALLAS_CITY = "21324"
DALTON_CITY = "21380"
DAMASCUS_CITY = "21436"
DANIELSVILLE_CITY = "21632"
DANVILLE_TOWN = "21688"
DARIEN_CITY = "21716"
DASHER_TOWN = "21744"
DAVISBORO_CITY = "21800"
DAWSON_CITY = "21912"
DAWSONVILLE_CITY = "21940"
DE_SOTO_CITY = "22472"
DEARING_TOWN = "22024"
DECATUR_CITY = "22052"
DEENWOOD_CDP = "22080"
DEEPSTEP_TOWN = "22108"
DEMOREST_CITY = "22304"
DENTON_CITY = "22388"
DEWY_ROSE_CDP = "22724"
DEXTER_TOWN = "22752"
DILLARD_CITY = "22976"
DIXIE_CDP = "23060"
DIXIE_UNION_CDP = "23144"
DOCK_JUNCTION_CDP = "23200"
DOERUN_CITY = "23284"
DONALSONVILLE_CITY = "23368"
DOOLING_TOWN = "23480"
DORAVILLE_CITY = "23536"
DOUGLAS_CITY = "23872"
DOUGLASVILLE_CITY = "23900"
DRUID_HILLS_CDP = "24264"
DU_PONT_TOWN = "24796"
DUBLIN_CITY = "24376"
DUDLEY_CITY = "24488"
DULUTH_CITY = "24600"
DUNWOODY_CITY = "24768"
DUTCH_ISLAND_CDP = "24876"
EAGLE_GROVE_CDP = "24964"
EAST_DUBLIN_CITY = "25300"
EAST_ELLIJAY_CITY = "25356"
EAST_GRIFFIN_CDP = "25440"
EAST_NEWNAN_CDP = "25692"
EAST_POINT_CITY = "25720"
EASTMAN_CITY = "25552"
EATONTON_CITY = "26084"
ECHOLS_COUNTY_CONSOLIDATED_GOVERNMENT = "26156"
EDGE_HILL_CITY = "26224"
EDISON_CITY = "26448"
ELBERTON_CITY = "26616"
ELLAVILLE_CITY = "26980"
ELLENTON_TOWN = "27008"
ELLERSLIE_CDP = "27064"
ELLIJAY_CITY = "27092"
EMERSON_CITY = "27344"
EMPIRE_CDP = "27512"
ENIGMA_TOWN = "27596"
EPHESUS_CITY = "27708"
EPWORTH_CDP = "27736"
ETON_CITY = "27932"
EUHARLEE_CITY = "27988"
EULONIA_CDP = "28016"
EVANS_CDP = "28044"
EVERETT_CDP = "28100"
EXPERIMENT_CDP = "28296"
FACEVILLE_CDP = "28324"
FAIR_OAKS_CDP = "28520"
FAIRBURN_CITY = "28380"
FAIRFIELD_PLANTATION_CDP = "28440"
FAIRMOUNT_CITY = "28492"
FAIRPLAY_CDP = "28548"
FAIRVIEW_CDP = "28632"
FARGO_CITY = "28744"
FAYETTEVILLE_CITY = "28968"
FITZGERALD_CITY = "29528"
FLEMINGTON_CITY = "30088"
FLOVILLA_CITY = "30312"
FLOWERY_BRANCH_CITY = "30340"
FOLKSTON_CITY = "30424"
FOREST_PARK_CITY = "30536"
FORSYTH_CITY = "30732"
FORT_GAINES_CITY = "30816"
FORT_OGLETHORPE_CITY = "30956"
FORT_STEWART_CDP = "31068"
FORT_VALLEY_CITY = "31096"
FRANKLIN_SPRINGS_CITY = "31320"
FRANKLIN_CITY = "31292"
FUNSTON_CITY = "31684"
GAINESVILLE_CITY = "31908"
GARDEN_CITY_CITY = "32048"
GARDI_CDP = "32132"
GARFIELD_CITY = "32188"
GAY_TOWN = "32384"
GENEVA_TOWN = "32412"
GEORGETOWN_CDP = "32482"
GEORGETOWN_QUITMAN_COUNTY_UNIFIED_GOVERNMENT = "32528"
GIBSON_CITY = "32748"
GILLSVILLE_CITY = "32860"
GIRARD_TOWN = "32916"
GLENN_CDP = "33280"
GLENNVILLE_CITY = "33336"
GLENWOOD_CITY = "33392"
GODFREY_CDP = "33672"
GOOD_HOPE_CITY = "33896"
GORDON_CITY = "33980"
GOUGH_CDP = "34176"
GRAHAM_CITY = "34260"
GRANTVILLE_CITY = "34428"
GRAY_CITY = "34512"
GRAYSON_CITY = "34596"
GREENSBORO_CITY = "34876"
GREENVILLE_CITY = "35016"
GRESHAM_PARK_CDP = "35240"
GRIFFIN_CITY = "35324"
GROVETOWN_CITY = "35716"
GUMBRANCH_CITY = "35800"
GUMLOG_CDP = "35814"
GUYTON_CITY = "35884"
HAGAN_CITY = "36024"
HAHIRA_CITY = "36052"
HAMILTON_CITY = "36220"
HAMPTON_CITY = "36276"
HANNAHS_MILL_CDP = "36416"
HAPEVILLE_CITY = "36472"
HARALSON_TOWN = "36528"
HARDWICK_CDP = "36668"
HARLEM_CITY = "36696"
HARRISON_TOWN = "36920"
HARTWELL_CITY = "37144"
HAWKINSVILLE_CITY = "37396"
HAZLEHURST_CITY = "37564"
HELEN_CITY = "37788"
HENDERSON_CDP = "37910"
HEPHZIBAH_CITY = "38040"
HERON_BAY_CDP = "38105"
HIAWASSEE_CITY = "38124"
HICKOX_CDP = "38292"
HIGGSTON_TOWN = "38348"
HILLTOP_CDP = "38866"
HILTONIA_TOWN = "38920"
HINESVILLE_CITY = "38964"
HIRAM_CITY = "39076"
HOBOKEN_CITY = "39216"
HOGANSVILLE_CITY = "39244"
HOLLY_SPRINGS_CITY = "39524"
HOMELAND_CITY = "39692"
HOMER_TOWN = "39720"
HOMERVILLE_CITY = "39748"
HORTENSE_CDP = "40028"
HOSCHTON_CITY = "40056"
HOWARD_CDP = "40196"
HULL_CITY = "40532"
IDEAL_CITY = "40812"
ILA_CITY = "40840"
INDIAN_SPRINGS_CDP = "41036"
IRON_CITY_TOWN = "41344"
IRONDALE_CDP = "41347"
IRWINTON_CITY = "41400"
ISLE_OF_HOPE_CDP = "41484"
IVEY_CITY = "41540"
JACKSON_CITY = "41596"
JACKSONVILLE_CITY = "41708"
JAKIN_CITY = "41792"
JASPER_CITY = "41932"
JEFFERSON_CITY = "41988"
JEFFERSONVILLE_CITY = "42100"
JEKYLL_ISLAND_CDP = "42128"
JENKINSBURG_CITY = "42156"
JERSEY_CITY = "42184"
JESUP_CITY = "42268"
JOHNS_CREEK_CITY = "42425"
JONESBORO_CITY = "42604"
JULIETTE_CDP = "42772"
JUNCTION_CITY_TOWN = "42800"
KENNESAW_STATE_UNIVERSITY_CDP = "43210"
KENNESAW_CITY = "43192"
KEYSVILLE_CITY = "43360"
KINGS_BAY_BASE_CDP = "43580"
KINGSLAND_CITY = "43640"
KINGSTON_CITY = "43668"
KITE_CITY = "43892"
KNOXVILLE_CDP = "44060"
LAFAYETTE_CITY = "44312"
LAGRANGE_CITY = "44340"
LAKE_CITY_CITY = "44508"
LAKE_PARK_CITY = "44704"
LAKELAND_CITY = "44592"
LAKEVIEW_CDP = "44900"
LAKEVIEW_ESTATES_CDP = "44956"
LAVONIA_CITY = "45460"
LAWRENCEVILLE_CITY = "45488"
LEARY_CITY = "45600"
LEESBURG_CITY = "45768"
LENOX_TOWN = "45936"
LESLIE_CITY = "46020"
LEXINGTON_CITY = "46188"
LILBURN_CITY = "46356"
LILLY_CITY = "46384"
LINCOLN_PARK_CDP = "46524"
LINCOLNTON_CITY = "46552"
LINDALE_CDP = "46580"
LITHIA_SPRINGS_CDP = "46832"
LITHONIA_CITY = "46860"
LOCUST_GROVE_CITY = "47140"
LOGANVILLE_CITY = "47196"
LONE_OAK_TOWN = "47252"
LOOKOUT_MOUNTAIN_CITY = "47336"
LOUISVILLE_CITY = "47560"
LOVEJOY_CITY = "47616"
LUDOWICI_CITY = "47784"
LULA_CITY = "47896"
LUMBER_CITY_CITY = "47952"
LUMPKIN_CITY = "47980"
LUTHERSVILLE_CITY = "48008"
LYERLY_TOWN = "48092"
LYONS_CITY = "48232"
MABLETON_CDP = "48288"
MACON_BIBB_COUNTY = "49008"
MADISON_CITY = "49196"
MANASSAS_CITY = "49504"
MANCHESTER_CITY = "49532"
MANOR_CDP = "49588"
MANSFIELD_CITY = "49616"
MARIETTA_CITY = "49756"
MARSHALLVILLE_CITY = "49924"
MARTIN_TOWN = "50008"
MARTINEZ_CDP = "50036"
MATTHEWS_CDP = "50176"
MAXEYS_TOWN = "50260"
MAYSVILLE_TOWN = "50484"
MCCAYSVILLE_CITY = "48428"
MCDONOUGH_CITY = "48624"
MCINTYRE_TOWN = "48848"
MCRAE_HELENA_CITY = "49100"
MEANSVILLE_CITY = "50596"
MEIGS_CITY = "50680"
MENDES_CDP = "50820"
MENLO_CITY = "50848"
MERSHON_CDP = "50960"
METTER_CITY = "51072"
MIDVILLE_CITY = "51240"
MIDWAY_CITY = "51352"
MILAN_CITY = "51408"
MILLEDGEVILLE_CITY = "51492"
MILLEN_CITY = "51520"
MILLWOOD_CDP = "51576"
MILNER_CITY = "51604"
MILSTEAD_CDP = "51660"
MILTON_CITY = "51670"
MINERAL_BLUFF_CDP = "51744"
MITCHELL_TOWN = "51968"
MOLENA_CITY = "52108"
MONROE_CITY = "52192"
MONTEZUMA_CITY = "52304"
MONTGOMERY_CDP = "52332"
MONTICELLO_CITY = "52416"
MONTROSE_TOWN = "52500"
MOODY_AFB_CDP = "52556"
MORELAND_TOWN = "52668"
MORGAN_CITY = "52696"
MORGANTON_CITY = "52724"
MORROW_CITY = "53004"
MORVEN_CITY = "53032"
MOULTRIE_CITY = "53060"
MOUNT_AIRY_TOWN = "53284"
MOUNT_VERNON_CITY = "53508"
MOUNT_ZION_CITY = "53620"
MOUNTAIN_CITY_TOWN = "53116"
MOUNTAIN_PARK_CDP = "53186"
MOUNTAIN_PARK_CITY = "53172"
MUSELLA_CDP = "54012"
NAHUNTA_CITY = "54124"
NASHVILLE_CITY = "54264"
NAYLOR_CDP = "54404"
NELSON_CITY = "54572"
NEW_ENGLAND_CDP = "54768"
NEWBORN_TOWN = "54656"
NEWINGTON_TOWN = "54992"
NEWNAN_CITY = "55020"
NEWTON_CITY = "55244"
NICHOLLS_CITY = "55440"
NICHOLSON_CITY = "55468"
NORCROSS_CITY = "55776"
NORMAN_PARK_CITY = "55860"
NORRISTOWN_CDP = "55944"
NORTH_DECATUR_CDP = "56112"
NORTH_DRUID_HILLS_CDP = "56168"
NORTH_HIGH_SHOALS_TOWN = "56308"
NORWOOD_CITY = "56560"
NUNEZ_CITY = "56644"
OAK_PARK_TOWN = "57232"
OAKWOOD_CITY = "57260"
OCHLOCKNEE_TOWN = "57372"
OCILLA_CITY = "57428"
OCONEE_CITY = "57456"
ODUM_CITY = "57540"
OFFERMAN_CITY = "57568"
OGLETHORPE_CITY = "57736"
OHOOPEE_CDP = "57876"
OLIVER_CITY = "58100"
OMEGA_CITY = "58184"
ORCHARD_HILL_TOWN = "58296"
OXFORD_CITY = "58744"
PALMETTO_CITY = "58884"
PANTHERSVILLE_CDP = "59080"
PARROTT_TOWN = "59416"
PATTERSON_CITY = "59500"
PAVO_CITY = "59556"
PEACHTREE_CITY_CITY = "59724"
PEACHTREE_CORNERS_CITY = "59735"
PEARSON_CITY = "59808"
PELHAM_CITY = "59976"
PEMBROKE_CITY = "60004"
PENDERGRASS_TOWN = "60032"
PERKINS_CDP = "60312"
PERRY_CITY = "60340"
PHILLIPSBURG_CDP = "60508"
PINE_LAKE_CITY = "61040"
PINE_MOUNTAIN_TOWN = "61124"
PINEHURST_CITY = "60984"
PINEVIEW_TOWN = "61320"
PINEY_GROVE_CDP = "61376"
PITTS_CITY = "61544"
PLAINS_CITY = "61628"
PLAINVILLE_CITY = "61684"
POOLER_CITY = "62104"
PORT_WENTWORTH_CITY = "62328"
PORTAL_TOWN = "62216"
PORTERDALE_CITY = "62244"
POULAN_CITY = "62496"
POWDER_SPRINGS_CITY = "62524"
PULASKI_TOWN = "63000"
PUTNEY_CDP = "63084"
QUITMAN_CITY = "63224"
RANGER_TOWN = "63560"
RAOUL_CDP = "63588"
RAY_CITY_CITY = "63728"
RAYLE_TOWN = "63756"
REBECCA_TOWN = "63840"
REDAN_CDP = "63952"
REED_CREEK_CDP = "64288"
REGISTER_TOWN = "64372"
REIDSVILLE_CITY = "64512"
REMERTON_CITY = "64596"
RENTZ_CITY = "64680"
RESACA_TOWN = "64736"
REST_HAVEN_TOWN = "64792"
REYNOLDS_CITY = "64876"
RHINE_TOWN = "64932"
RICEBORO_CITY = "64960"
RICHLAND_CITY = "65016"
RICHMOND_HILL_CITY = "65044"
RIDDLEVILLE_TOWN = "65156"
RINCON_CITY = "65296"
RINGGOLD_CITY = "65324"
RIVERDALE_CITY = "65464"
ROBERTA_CITY = "65856"
ROBINS_AFB_CDP = "65968"
ROCHELLE_CITY = "66024"
ROCK_SPRING_CDP = "66304"
ROCKINGHAM_CDP = "66220"
ROCKMART_CITY = "66276"
ROCKY_FORD_TOWN = "66444"
ROME_CITY = "66668"
ROOPVILLE_TOWN = "66696"
ROSSVILLE_CITY = "67256"
ROSWELL_CITY = "67284"
ROYSTON_CITY = "67452"
RUSSELL_CDP = "67676"
RUTLEDGE_CITY = "67760"
SALE_CITY_CITY = "68096"
SALEM_CDP = "68129"
SANDERSVILLE_CITY = "68208"
SANDY_SPRINGS_CITY = "68516"
SANTA_CLAUS_CITY = "68600"
SARDIS_CITY = "68740"
SASSER_TOWN = "68796"
SATILLA_CDP = "68805"
SAUTEE_NACOOCHEE_CDP = "68880"
SAVANNAH_CITY = "69000"
SCOTLAND_CITY = "69336"
SCOTTDALE_CDP = "69392"
SCREVEN_CITY = "69448"
SENOIA_CITY = "69672"
SEVILLE_CDP = "69728"
SHADY_DALE_TOWN = "69784"
SHANNON_CDP = "69868"
SHARON_CITY = "69896"
SHARPSBURG_TOWN = "69980"
SHELLMAN_CITY = "70120"
SHILOH_CITY = "70428"
SILOAM_TOWN = "70932"
SKIDAWAY_ISLAND_CDP = "71184"
SKY_VALLEY_CITY = "71282"
SMARR_CDP = "71296"
SMITHVILLE_CITY = "71464"
SMYRNA_CITY = "71492"
SNELLVILLE_CITY = "71604"
SOCIAL_CIRCLE_CITY = "71660"
SOPERTON_CITY = "71772"
SOUTH_FULTON_CITY = "72122"
SPARKS_TOWN = "72556"
SPARTA_CITY = "72584"
SPRINGFIELD_CITY = "72780"
ST_MARYS_CITY = "67984"
ST_SIMONS_CDP = "68040"
STAPLETON_CITY = "73004"
STATESBORO_CITY = "73256"
STATHAM_CITY = "73312"
STERLING_CDP = "73452"
STILLMORE_CITY = "73620"
STOCKBRIDGE_CITY = "73704"
STOCKTON_CDP = "73760"
STONE_MOUNTAIN_CITY = "73816"
STONECREST_CITY = "73784"
SUGAR_HILL_CITY = "74180"
SUMMERTOWN_CITY = "74348"
SUMMERVILLE_CITY = "74376"
SUMNER_TOWN = "74432"
SUNNY_SIDE_CITY = "74572"
SUNNYSIDE_CDP = "74628"
SUNSET_VILLAGE_CDP = "74768"
SURRENCY_TOWN = "74852"
SUWANEE_CITY = "74936"
SWAINSBORO_CITY = "74964"
SYCAMORE_CITY = "75104"
SYLVANIA_CITY = "75160"
SYLVESTER_CITY = "75188"
TALAHI_ISLAND_CDP = "75239"
TALBOTTON_CITY = "75244"
TALKING_ROCK_TOWN = "75272"
TALLAPOOSA_CITY = "75300"
TALLULAH_FALLS_TOWN = "75328"
TALMO_TOWN = "75412"
TARRYTOWN_TOWN = "75496"
TATE_CITY_CDP = "75580"
TAYLORSVILLE_TOWN = "75664"
TAZEWELL_CDP = "75692"
TEMPLE_CITY = "75832"
TENNILLE_CITY = "75888"
THE_ROCK_CDP = "76112"
THOMASTON_CITY = "76168"
THOMASVILLE_CITY = "76224"
THOMSON_CITY = "76280"
THUNDERBOLT_TOWN = "76364"
TIFTON_CITY = "76476"
TIGER_TOWN = "76504"
TIGNALL_TOWN = "76532"
TOCCOA_CITY = "76756"
TOOMSBORO_CITY = "76952"
TRENTON_CITY = "77372"
TRION_TOWN = "77540"
TUCKER_CITY = "77652"
TUNNEL_HILL_CITY = "77736"
TURIN_TOWN = "77764"
TWIN_CITY_CITY = "77988"
TY_TY_CITY = "78100"
TYBEE_ISLAND_CITY = "78036"
TYRONE_TOWN = "78044"
UNADILLA_CITY = "78156"
UNION_CITY_CITY = "78324"
UNION_POINT_CITY = "78380"
UNIONVILLE_CDP = "78464"
UVALDA_CITY = "78744"
VALDOSTA_CITY = "78800"
VARNELL_CITY = "78996"
VERNONBURG_TOWN = "79164"
VIDALIA_CITY = "79388"
VIDETTE_CITY = "79416"
VIENNA_CITY = "79444"
VILLA_RICA_CITY = "79528"
VININGS_CDP = "79612"
WACO_CITY = "79808"
WADLEY_CITY = "79864"
WALESKA_CITY = "79948"
WALNUT_GROVE_CITY = "80228"
WALTHOURVILLE_CITY = "80256"
WARESBORO_CDP = "80368"
WARM_SPRINGS_CITY = "80480"
WARNER_ROBINS_CITY = "80508"
WARRENTON_CITY = "80592"
WARTHEN_CDP = "80648"
WARWICK_CITY = "80676"
WASHINGTON_CITY = "80704"
WATKINSVILLE_CITY = "80788"
WAVERLY_CDP = "80816"
WAVERLY_HALL_TOWN = "80844"
WAYCROSS_CITY = "80956"
WAYNESBORO_CITY = "80984"
WAYNESVILLE_CDP = "81012"
WEBSTER_COUNTY_UNIFIED_GOVERNMENT = "81128"
WENONA_CDP = "81208"
WEST_BROW_CDP = "81488"
WEST_POINT_CITY = "82132"
WHIGHAM_CITY = "82412"
WHITE_PLAINS_CITY = "82692"
WHITE_CITY = "82468"
WHITEMARSH_ISLAND_CDP = "82636"
WHITESBURG_CITY = "82720"
WILDWOOD_CDP = "82916"
WILLACOOCHEE_CITY = "82972"
WILLIAMSON_CITY = "83084"
WILMINGTON_ISLAND_CDP = "83168"
WINDER_CITY = "83420"
WINTERVILLE_CITY = "83728"
WOODBINE_CITY = "83868"
WOODBURY_CITY = "83896"
WOODLAND_CITY = "83952"
WOODSTOCK_CITY = "84176"
WOODVILLE_CITY = "84260"
WOOLSEY_TOWN = "84288"
WRENS_CITY = "84456"
WRIGHTSVILLE_CITY = "84512"
YATESVILLE_TOWN = "84708"
YONAH_CDP = "84832"
YOUNG_HARRIS_CITY = "84960"
ZEBULON_CITY = "85128"
